use std::fs;
use mailparse::parse_mail;
use super::util::{parse_email, xml_safe as x};
use crate::models::*;
use scraper::{Html as scraper_html, Selector};

fn message(
    title: &str,
    entry_id: &str,
    updated: &str,
    author_name: &str,
    author_email: &str,
    content: &str,
) -> String {
    format!(
        r#"
        <?xml version="1.0" encoding="utf-8"?>
        <entry>
            <title>{title}</title>
            <id>{entry_id}</id>
            <updated>{updated}</updated>
            <author>
                <name>{author_name}</name>
                <email>{author_email}</email>
            </author>
            <content type="text/plain">
                {content}
            </content>
        </entry>
        "#,
    )
}

impl StrMessage {
    pub fn to_xml(&self) -> String {
        let msg = self;

        let data = match fs::read_to_string(self.original_path.clone()) {
            Ok(content) => content,
            Err(e) => {
                eprintln!("Error reading file: {}", e);
                return "Error loading content".to_string();
            }
        };
        
        let parsed_mail = match parse_mail(&data.as_bytes()) {
            Ok(mail) => mail,
            Err(e) => return format!("Error parsing email: {}", e),
        };
        
        let html = parse_email(&parsed_mail).unwrap_or_else(|_| msg.body.clone());
        
        let document = scraper_html::parse_document(&*html);
        let body = Selector::parse("body").unwrap();
        
        // Traverse the parsed HTML and extract all text from the body
        let text = document.select(&body)
            .next().unwrap()
            .text()
            .collect::<Vec<_>>()
            .join(" ");
        
        // TODO use xml_safe
        message(
            &x(&msg.subject),
            &x(&msg.id),
            &msg.received.to_rfc3339(),
            &x(&msg.from.clone().name.unwrap_or(msg.from.clone().address)),
            &x(&msg.from.address),
            &x(&text),
        )
    }
}
